class ReactModel:
  """
  A React Model handles the interaction between your application
  and its external SQL relational database tables.
  """
  
  def new(self, **kwargs):
    """
    Creates a new React Model object. Properties on the object represent
    columns of the database record. Arguments passed to this method are set
    as propertis on the object.
    
    Examples::
    
      thing1 = models.things.new()
      thing1.name = "My First Thing"
      thing1.widgets = 24
      
      thing2 = models.things.new(name="My Second Thing", widgets=25)
    """

  def find_all(self, conditions=None):
    """
    Finds and returns all records (React Model objects) matching 'conditions'
    (if specified)
    
    Arguments:
    
    conditions -- either a single string or a list of strings to match as
      the WHERE clause of the SQL query; if a list of strings is passed,
      they are joined together using "AND"
    """
  
  def find(self, id):
    """
    Returns the React Model object with the id specified
    
    Arguments:
    
    id -- the integer id of the record
    """
  
  def find_by_sql(self, sql):
    """
    Using the SQL query string provided, query the database and return the
    first record (React Model object) found.
    
    Arguments:
    
    sql -- the SQL query string to use
    
    Example::
    
      thing = models.things.find_by_sql("name = 'My Thing'")
    """
  
  def find_by_value(self, column, value):
    """
    Find and return the record with 'column' matching 'value'.
    
    Arguments:
    
    column -- the column to lookup
      
    value -- the value to match
      
    Example::
    
      thing = models.things.find_by_value("name", "My Thing")
    """

class ReactModelRecord:
  """
  Represents a single database table record, i.e. React Model object.
  Properties on this object represent columns of the database table.
  Changing these properties do not affect the database record until
  the 'save()' method is called.
  """
  
  def save(self):
    """
    Saves a React Model object (record) to the database table
    with which it is associated. If this object has an id,
    then do an update to an existing record; if not, insert
    a new record.
    """
  
  def destroy(self):
    """
    Deletes the record from the database. This operation is
    immediate and cannot be undone.
    """
